@@ -248,6 +248,9 @@ class EbookViewer(MainWindow):
             QTimer.singleShot(50, file_events.flush)
         self.window_mode_changed = None
         self.toggle_toolbar_action = QAction(_('Show/hide controls'), self)
+        tts = self.view.shortcuts.get_shortcuts('Show/hide controls')
+        if tts:
+            self.toggle_toolbar_action.setText(self.toggle_toolbar_action.text() + '\t' + tts[0])
         self.toggle_toolbar_action.setCheckable(True)
         self.toggle_toolbar_action.triggered.connect(self.toggle_toolbars)
         self.toolbar_hidden = None
@@ -261,7 +264,7 @@ class EbookViewer(MainWindow):
         self.clear_recent_history_action = QAction(
                 _('Clear list of recently opened books'), self)
         self.clear_recent_history_action.triggered.connect(self.clear_recent_history)
-        self.build_recent_menu()
+        self.open_history_menu.aboutToShow.connect(self.build_recent_menu)
         self.open_history_menu.triggered.connect(self.open_recent)
 
         for x in ('tool_bar', 'tool_bar2'):
@@ -329,7 +332,6 @@ class EbookViewer(MainWindow):
 
     def clear_recent_history(self, *args):
         vprefs.set('viewer_open_history', [])
-        self.build_recent_menu()
 
     def build_recent_menu(self):
         m = self.open_history_menu
@@ -347,9 +349,11 @@ class EbookViewer(MainWindow):
                 count += 1
 
     def continue_reading(self):
-        actions = self.open_history_menu.actions()[2:]
-        if actions:
-            actions[0].trigger()
+        recent = vprefs.get('viewer_open_history', [])
+        if recent:
+            for path in recent:
+                if os.path.exists(path):
+                    self.load_ebook(path)
 
     def shutdown(self):
         if self.isFullScreen() and not self.view.document.start_in_fullscreen:
@@ -1007,7 +1011,6 @@ class EbookViewer(MainWindow):
                 except Exception:
                     import traceback
                     traceback.print_exc()
-            self.build_recent_menu()
             self.view.set_book_data(self.iterator)
 
             self.footnotes_dock.close()
@@ -1107,12 +1110,15 @@ class EbookViewer(MainWindow):
             'Reload': self.action_reload,
             'Table of Contents': self.action_table_of_contents,
             'Print': self.action_print,
+            'Show/hide controls': self.toggle_toolbar_action,
         }.get(key, None)
         if action is not None:
             event.accept()
             action.trigger()
             return
         if key == 'Focus Search':
+            if not self.tool_bar.isVisible():
+                self.toggle_toolbars()
             self.search.setFocus(Qt.OtherFocusReason)
             return
         if not self.view.handle_key_press(event):
